<?php
/*
Plugin Name: Filosofo's Tinfoil Hat Plugin
Plugin URI: http://www.ilfilosofo.com/blog/tinfoil-hat/
Description: Check for plugin and WordPress updates without sending the list of all your plugins associated with your URL.
Author: Austin Matzko
Version: 1.0.2
Author URI: http://www.ilfilosofo.com/
*/

class filosofo_tinfoil_hat {
	var $td = 'filosofo-tinfoil-hat';
	var $check_interval;
	var $debug = false;
	var $debug_data = '';

	function filosofo_tinfoil_hat() {
		register_activation_hook(__FILE__, array(&$this, 'schedule_all_plugin_checks'));
		add_action('admin_menu', array(&$this, 'remove_default_actions'));
		add_action('after_plugin_row', array(&$this, 'plugin_update_row'));
		add_filter('cron_schedules', array(&$this, 'add_sched_options'));
		add_action('init', array(&$this, 'init'));
		add_action('init', array(&$this, 'version_check'));
		add_action('load-plugins.php', array(&$this, 'load_plugins'));
		add_action('plugins_loaded', array(&$this, 'remove_default_actions'));
		add_action('tinfoil_check_wp_version', array(&$this, 'version_check'));
		add_action('tinfoil_update_plugin', array(&$this, 'plugin_update'));
		add_action('update_option_active_plugins', array(&$this, 'schedule_all_plugin_checks'),999);

		$this->check_interval = apply_filters('filosofo_tinfoil_check_interval', 'twiceaday');
	}

	function add_sched_options($sched) {
		$sched['twiceaday'] = array('interval' => 43200, 'display' => __('Twice a Day',$this->td));
		return $sched;
	}

	function debug() {
		$b = "\n\n#################################################################\n\n";
		$plugin_data = get_option('update_plugins');
		$my_plugin_list = get_option('filosofo_tinfoil_plugins');
		echo '<pre>';
		echo $b . __('Debug: Internal plugin update data:', $this->td);
		print_r($plugin_data);
		echo $b . __('Debug: List of plugins:', $this->td);
		print_r($my_plugin_list);
		
		// try a sample lookup
		$this->plugin_update($this->get_plugin_slug('akismet/akismet.php'));
		echo $b . __('Debug: General data:', $this->td);
		echo $this->debug_data . $b;
		echo '</pre>';
		return;
	}
		function debug_data( $data = null ) {
			$this->debug_data .= "\r\n" . print_r( $data, true ) . "\r\n";	
		}


	function check_api() {
		if ( ! current_user_can('activate_plugins') || ! is_admin() )
			return false;


	}

	function plugin_admin_header() {
		if ( $this->debug )
			$this->debug();
		?>
		<script type="text/javascript">
		//<![CDATA[
		if ( undefined != typeof(jQuery) ) { jQuery(function(j) {
			j('a.remind-link').bind('click',function() {
				var theLink = this;
				j.get( j(this).attr('href'), {tinfoilAjax: 1}, function(r){
					if ( parseInt(r,10) ) 
						j(theLink).parent().fadeOut('500');
				});
				return false;
			});
		});}
		//]]>
		</script>
		<?php
	}

	function get_plugin_slug($plugin = '') {	
		return 'tinfoil_update_' . sanitize_title($plugin);
	}

	function get_plugins($refresh = false) {
		$plugins = get_option('filosofo_tinfoil_plugins');
		if ( ( $refresh || empty( $plugins ) ) && function_exists('get_plugins') ) :
			$plugins = array();
			foreach( (array) get_plugins() as $plugin_file => $pdata )
				$plugins[$this->get_plugin_slug($plugin_file)] = array_merge((array) $pdata, array('file' => $plugin_file));
			update_option('filosofo_tinfoil_plugins', $plugins);
		endif;
		return $plugins;
	}
	
	function init() {
		load_plugin_textdomain($this->td);
		// check for remind
		if ( isset($_REQUEST['remind_later']) && isset($_REQUEST['tinfoil_update']) && is_admin() ) :
			$plugin = $_REQUEST['remind_later'];
			check_admin_referer('tinfoil-hat_remind_later');
			$current = get_option('update_plugins');
			if ( isset($current->checked[$plugin]) )
				unset($current->checked[$plugin]);
			if ( isset($current->response[$plugin]) )
				unset($current->response[$plugin]);
			$time = time() + 604800;
			update_option('update_plugins', $current);
			$this->schedule_plugin_check($this->get_plugin_slug($plugin), $time, $recurring = true );
			if ( isset($_REQUEST['tinfoilAjax']) ) : 
				die('1');
			endif;
		endif;
	}

	function load_plugins() {
		add_action('admin_head', array(&$this, 'plugin_admin_header'));
		wp_enqueue_script('jquery');
	}
	
	function make_request($args = array()) {
		$defs = array('client' => 'fsockopen', 'contenttype' => 'application/x-www-form-urlencoded; charset=' . get_option('blog_charset'), 'host' => 'api.wordpress.org', 'method' => 'POST', 'path' => '', 'port' => '80', 'request' => array(), 'sitehome' => '', 'useragent' => 'WordPress/');
		$args = array_merge($defs, $args);
		extract($args, EXTR_SKIP);

		// build header
		$req_str = '';
		foreach( (array) $request as $key => $value ) 
			$req_str .= $key . '=' . urlencode( $value ) . '&';

		$req_str = substr($req_str,'',-1);
		$req = "$method $path";
		if ( 'GET' == $method ) 
			$req .= '?' . $req_str;
		$req .= " HTTP/1.0\r\n";
		$req .= "Host: $host\r\n";
		$req .= "Content-Type: $contenttype\r\n";
		$req .= "User-Agent: {$useragent}; $sitehome \r\n";
		if ( 'POST' == $method ) :
			$req .= "Content-Length: " . strlen($req_str) . "\r\n";
			$req .= "\r\n$req_str";
		endif;
		$req .= "\r\n";

		if ( $this->debug ) 
			$this->debug_data( $req );

		// use fsockopen if available
		if ( function_exists('fsockopen') && 'fsockopen' == $client ) :
			
			if ( $this->debug )
				$this->debug_data(__('Using fsockopen', $this->td));

			$response = '';
			if ( false != ( $fs = @fsockopen( $host, $port, $errno, $errstr, 3) ) && is_resource($fs) ) :
				fwrite($fs, $req);

				while ( !feof($fs) )
					$response .= fgets($fs, 1160); // One TCP-IP packet
				fclose($fs);
				$response = explode("\r\n\r\n", $response, 2);
				$response = $response[1];
			endif;
	
		// or use stream_context_create
		elseif ( function_exists('stream_context_create') ) :
			if ( $this->debug )
				$this->debug_data(__('Using stream_context_create', $this->td));

			$response = '';
			$params = array('http' => array( 'method' => $method, 'header' => $req ));

			$context = @stream_context_create($params);
			if ( false != ( $fs = fopen( 'http://' . $host . $path, 'r', false, $context) ) && is_resource($fs) ) :
				while ( !feof($fs) )
					$response .= fgets($fs, 1160); // One TCP-IP packet
				fclose($fs);
			endif;

		// or use cURL
		elseif ( function_exists('curl_init') ) :
			if ( $this->debug )
				$this->debug_data(__('Using cURL', $this->td));

			$curl = curl_init();
			curl_setopt($curl, CURLOPT_PORT, $port);
			if ($method == 'POST') {
				curl_setopt($curl, CURLOPT_POST, 1);
				curl_setopt($curl, CURLOPT_POSTFIELDS, $req_str);
			}
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_URL, 'http://' . $host . $path);
			curl_setopt($curl, CURLOPT_USERAGENT, $useragent);
			$response = @curl_exec($curl);
			curl_close($curl);

		endif;
		return $response;
	}

	function plugin_update( $pluginhook = '' ) {
		global $wp_version;

		$plugins = $this->get_plugins();
		$data = $plugins[$pluginhook];
		
		if ( empty( $data ) )
			return false;
		
		$file = $data['file'];
		
		$current = get_option( 'update_plugins' );

		$current = ( empty( $current ) ) ? '' : $current;

		$current->checked[ $file ] = $data['Version'];

		$version = ( version_compare( $data['Version'], '0.1', '<' ) ) ? $data['Version'] : '0.1';

		$to_send->active = array();
		$to_send->plugins[$file] = array(
			'Name' => $data['Name'],
			'Title' => '',
			'Description' => '',
			'Author' => '',
			'Version' => $version,
		);
	
		if ( $this->debug ) 
			$this->debug_data( $to_send );

		$request = serialize( $to_send );
		$response = $this->make_request(array('request' => array('plugins' => $request), 'method' => 'POST', 'path' => '/plugins/update-check/1.0/', 'sitehome' => wp_hash(get_option('home') . $file) ));

		if ( $this->debug )
			$this->debug_data( $response );


		if ( ! is_array($response) ) :
			$response = unserialize( $response );
		
			if ( $this->debug ) {
				$this->debug_data(__('Response:', $this->td));
				$this->debug_data($response);
			}
		endif;

		if ( is_array( $response ) && ( ! empty( $response ) ) ) :
			$current->response[$file] = $response[$file];
		else :
			if ( $this->debug )
				$this->debug_data(__('No meaningful response', $this->td));

			if ( isset( $current->checked[$file] ) ) :
				unset( $current->checked[$file] );
			endif;

			if ( isset( $current->response[$file] ) ) :
				unset( $current->response[$file] );
			endif;
		endif;
		update_option( 'update_plugins', $current );
	}

	function plugin_update_row( $file ) {
		global $plugin_data;
		$current = get_option( 'update_plugins' );
		$remind_link = wp_nonce_url(add_query_arg(array('remind_later' => $file, 'tinfoil_update' => 1), get_bloginfo('wpurl') . '/wp-admin/plugins.php'), 'tinfoil-hat_remind_later');

		if ( !isset( $current->response[ $file ] ) )
			return false;
		
		if ( ! version_compare($plugin_data['Version'], $current->response[$file]->new_version, '<') )
			return false;

		$r = $current->response[ $file ];

		?><tr><td colspan='5' class='plugin-update'><?php
		printf( __('There is a new version of %s available. <a href="%s">Download version %s here</a>.', $this->td), $plugin_data['Name'], $r->url, $r->new_version ); ?>
		<?php
		printf( __('<a href="%s" class="remind-link">Remind me again next week.</a>', $this->td), $remind_link);
		?></td></tr><?php
	}

	function remove_default_actions() {
		remove_action('after_plugin_row','wp_plugin_update_row');
		remove_action('init','wp_version_check');
		remove_action('load-plugins.php','wp_update_plugins');
	}

	function schedule_plugin_check($plugin = '', $time = '', $recurring = true ) {
		if ( empty( $plugin ) ) 
			return false;
		$spacing = rand(1,72) * 600; // schedule the plugin check to start some time in the next 12 hours
		$time = ( empty( $time ) ) ? time() : $time;
		$time = $time + (int) $spacing;
		$next = wp_next_scheduled( 'tinfoil_update_plugin', array($plugin) );
		if ( ! empty( $next ) )
			wp_unschedule_event($next, 'tinfoil_update_plugin', array($plugin));
		if ( $recurring )
			wp_schedule_event($time, $this->check_interval, 'tinfoil_update_plugin', array($plugin));
		else 
			wp_schedule_single_event($time, 'tinfoil_update_plugin', array($plugin));
			
	}

	function schedule_version_check( $recurring = true ) {
		$time = time();
		$next = wp_next_scheduled( 'tinfoil_check_wp_version' );
		if ( ! empty( $next ) )
			wp_unschedule_event($next, 'tinfoil_check_wp_version');
		if ( $recurring )
			wp_schedule_event($time, $this->check_interval, 'tinfoil_check_wp_version');
		else 
			wp_schedule_single_event($time, 'tinfoil_check_wp_version');
	}

	function schedule_all_plugin_checks() {
		$this->schedule_version_check();
		$plugins = $this->get_plugins(true);
		foreach( (array) $plugins as $slug => $plugin ) :
			$this->schedule_plugin_check($slug);
		endforeach;
	}

	function version_check() {
		global $wp_version;
		if ( strpos($_SERVER['PHP_SELF'], 'install.php') !== false || defined('WP_INSTALLING') )
			return;

		$php_version = wp_hash(phpversion() . get_bloginfo('url'));

		$version = $wp_version;

		$current = get_option( 'update_core' );
		$locale = get_locale();
		
		$new_option = '';
		$new_option->version_checked = $version;
		
		$response = $this->make_request(array('method' => 'GET', 'request' => array('version' => $version, 'php' => $php_version, 'locale' => $locale), 'path' => '/core/version-check/1.0/', 'siteurl' => wp_hash(get_bloginfo('url')), 'useragent' => 'WordPress/'));

		if ( ! is_array( $response ) ) 
			$response = explode("\r\n\r\n", $response, 2);
		if ( 1 === count( $response ) )
			array_unshift( $response, '');
		
		$body = trim( $response[1] );
		$body = str_replace(array("\r\n", "\r"), "\n", $body);
		$returns = explode("\n", $body);
		
		$new_option->response = $returns[0];
		if ( isset( $returns[1] ) )
			$new_option->url = $returns[1];

		update_option( 'update_core', $new_option );
	}


}
$filosofo_tinfoil_hat = new filosofo_tinfoil_hat();
?>
